@@ -66,9 +66,7 @@ print.tdt <- function(x, digits=4, ...){
 tdt2way <- function(...)
 	cat("tdt2way has been renamed to tdtGxG. So please use tdtGxG instead.")
 
-tdtGxG <- function(snp1, snp2, epistatic=TRUE,
-		model=c("additive", "dominant", "recessive"), add=FALSE,
-		warnError=TRUE){
+tdtGxG <- function(snp1, snp2, test=c("epistatic", "lrt", "full", "screen"), model=c("additive", "dominant", "recessive")){
 	require(survival)
 	n1 <- length(snp1)
 	n2 <- length(snp2)
@@ -82,6 +80,7 @@ tdtGxG <- function(snp1, snp2, epistatic=TRUE,
 	if(any(!snp2 %in% c(0, 1, 2, NA)))
 		stop("The values in snp2 must be 0, 1, and 2.")
 	type <- match.arg(model)
+	testtype <- match.arg(test)
 	mat.trio <- cbind(matrix(snp1, ncol=3, byrow=TRUE), matrix(snp2, ncol=3, byrow=TRUE))
 	mat.trio <- mat.trio[rowSums(is.na(mat.trio))==0,]
 	mat <- matrix(c(0,0,0,0, 0,0,1,1, 0,0,1,1, 1,0,0,1, 1,0,0,1, 1,1,1,1, 1,1,1,1,
@@ -109,8 +108,8 @@ tdtGxG <- function(snp1, snp2, epistatic=TRUE,
 	n.trio <- length(code1)		
 	x1 <- as.vector(mat[,code1])
 	x2 <- as.vector(mat[,code2])
-	if(epistatic)
-		return(tdtEpistatic(x1, x2, n.trio, warnError=warnError))
+	if(testtype=="epistatic")
+		return(tdtEpistatic(x1, x2, n.trio))
 	if(type=="dominant"){
 		x1 <- (x1 >= 1) * 1
 		x2 <- (x2 >= 1) * 1
@@ -124,26 +123,37 @@ tdtGxG <- function(snp1, snp2, epistatic=TRUE,
 	IA <- x1[mat.ids[,1]] * x2[mat.ids[,2]]
 	y <- rep.int(c(1, rep.int(0, 15)), n.trio)
 	strat <- rep(1:n.trio, e=16)
-	if(warnError){
-		wa <- options()$warn
-		on.exit(options(warn=wa))
-		options(warn=2)
-	}
-	if(!add)
+	wa <- options()$warn
+	options(warn=2)
+	if(testtype=="screen")
 		c.out <- try(clogit(y ~ IA + strata(strat)), silent=TRUE)
 	else{
 		SNP1 <- x1[mat.ids[,1]]
 		SNP2 <- x2[mat.ids[,2]]
 		c.out <- try(clogit(y ~ SNP1 + SNP2 + IA + strata(strat)), silent=TRUE)
+		if(testtype=="lrt")
+			c.out2 <- try(clogit(y ~ SNP1 + SNP2 + strata(strat)), silent=TRUE)
+	}
+	options(warn=wa)
+	if(testtype=="lrt"){
+		ll.main <- if(is(c.out2, "try-error")) NA else c.out2$loglik[2]
+		ll.full <- if(is(c.out, "try-error")) NA else c.out$loglik[2]
+		if(!is.na(ll.full) && !is.na(ll.main)){
+			stat <- -2 * (ll.main - ll.full)
+			pval <- pchisq(stat, 4, lower.tail=FALSE)
+		}
+		else
+			stat <- pval <- NA
+		out <- list(ll.main=ll.main, ll.full=ll.full, stat=stat, pval=pval, 
+			full.model=c.out, testtype="lrt")
+		return(out)
 	}
-	if(warnError)
-		options(warn=wa)
 	if(is(c.out, "try-error"))
 		coef <- se <- stat <- pval <- lower <- upper <- NA
 	else{
 		coef <- c.out$coefficients
 		se <- sqrt(diag(c.out$var))
-		if(!add)
+		if(type=="screen")
 			names(coef) <- NULL
 		stat <- (coef/se)^2
 		pval <- pchisq(stat, 1, lower.tail=FALSE)
@@ -156,6 +166,7 @@ tdtGxG <- function(snp1, snp2, epistatic=TRUE,
 	out
 
 }
+
 	
 
 colTDT <- function(mat.snp, model=c("additive", "dominant", "recessive"), size=50){
@@ -177,9 +188,8 @@ colTDT2way <- function(...){
 	cat("colTDT2way has been renamed to colGxG. So please use colGxG instead.\n")
 }
 
-colGxG <- function(mat.snp, epistatic=TRUE, genes=NULL, maf=FALSE,
-		model=c("additive", "dominant", "recessive"), add=FALSE,
-		warnError=TRUE){
+colGxG <- function(mat.snp, test=c("epistatic", "lrt", "full", "screen"), genes=NULL, maf=FALSE,
+		model=c("additive", "dominant", "recessive")){
 	require(survival)
 	if(!is.matrix(mat.snp))
 		stop("mat.snp has to be a matrix.")
@@ -192,6 +202,7 @@ colGxG <- function(mat.snp, epistatic=TRUE, genes=NULL, maf=FALSE,
 	if(any(!mat.snp %in% c(0, 1, 2, NA)))
 		stop("The values in mat.snp must be 0, 1, and 2.")
 	type <- match.arg(model)
+	testtype <- match.arg(test)
 	mat.code <- matrix(c(0,0,0,0, 0,0,1,1, 0,0,1,1, 1,0,0,1, 1,0,0,1, 1,1,1,1, 
 		1,1,1,1, 2,2,2,2, 1,1,2,2, 1,1,2,2, 2,1,1,2, 2,1,1,2, 0,1,1,2, 
 		1,0,1,2, 2,0,1,1, NA,NA,NA,NA), nrow=4)
@@ -220,9 +231,8 @@ colGxG <- function(mat.snp, epistatic=TRUE, genes=NULL, maf=FALSE,
 	else
 		valMAF <- NULL
 	n.trio <- length(code)
-	if(epistatic)
-		return(colTDTepistatic(mat.pseudo, n.trio, colnames(mat.snp), genes=genes, valMAF=valMAF,
-			warnError=warnError))
+	if(testtype=="epistatic")
+		return(colTDTepistatic(mat.pseudo, n.trio, colnames(mat.snp), genes=genes, valMAF=valMAF))
 	if(type=="dominant")
 		mat.pseudo <- (mat.pseudo >= 1) * 1
 	if(type=="recessive")
@@ -244,11 +254,12 @@ colGxG <- function(mat.snp, epistatic=TRUE, genes=NULL, maf=FALSE,
 		combs <- allBetweenCombs(ids.genes)
 		coef <- se <- numeric(nrow(combs))
 	}
-	if(warnError){
-		wa <- options()$warn
-		on.exit(options(warn=wa))
-		options(warn=2)
-	}
+	if(testtype=="lrt")
+		return(colGxGlrt(mat.pseudo, mat.ids, combs, y, strat, rn=colnames(mat.snp), genes=genes,
+			valMAF=valMAF))
+	add <- testtype=="full"
+	wa <- options()$warn
+	options(warn=2)
 	for(i in 1:nrow(combs)){
 		x1 <- mat.pseudo[mat.ids[,1], combs[i,1]] 
 		x2 <- mat.pseudo[mat.ids[,2], combs[i,2]]
@@ -260,12 +271,10 @@ colGxG <- function(mat.snp, epistatic=TRUE, genes=NULL, maf=FALSE,
 		coef[i] <- if(is(c.out, "try-error")) NA else c.out$coefficients[1]
 		se[i] <- if(is(c.out, "try-error")) NA else sqrt(diag(c.out$var))[1]
 	}
-	if(warnError)
-		options(warn=wa)
+	options(warn=wa)
 	if(any(is.na(coef)))
-		warning("The fitting of some of the models has failed. A possible reason\n",
-			"is that the two SNPs might be in (strong) LD. For these interactions,\n",
-			"all statistics are therefore set to NA.", call.=FALSE)
+		warning("For at least one interaction,the fitting of the model has failed.\n",
+			"Therefore, all statistics for these interactions are set to NA.", call.=FALSE)
 	stat <- (coef / se)^2
 	lower <- exp(coef - qnorm(0.975) * se)
 	upper <- exp(coef + qnorm(0.975) * se)
@@ -293,6 +302,7 @@ colGxG <- function(mat.snp, epistatic=TRUE, genes=NULL, maf=FALSE,
 
 }
 
+
 print.colTDT <- function(x, top=5, digits=4, ...){
 	pval <- format.pval(x$pval, digits=digits)
 	if(x$ia){
@@ -320,7 +330,7 @@ print.colTDT <- function(x, top=5, digits=4, ...){
 	print(format(out, digits=digits))
 }
 
-tdtEpistatic <- function(x1, x2, n.trio, warnError=TRUE){
+tdtEpistatic <- function(x1, x2, n.trio){
 	x1 <- x1 - 1
 	z1 <- ifelse(x1==0, 0.5, -0.5)
 	x2 <- x2 - 1
@@ -339,8 +349,7 @@ tdtEpistatic <- function(x1, x2, n.trio, warnError=TRUE){
 	y <- rep.int(c(1, rep.int(0, 15)), n.trio)
 	strat <- rep(1:n.trio, e=16)
 	wa <- options()$warn
-	on.exit(options(warn=wa))
-	options(warn=ifelse(warnError, 2, wa))
+	options(warn=2)
 	woIA <- try(clogit(y ~ x1 + z1 + x2 + z2 + strata(strat)), silent=TRUE)
 	full <- try(clogit(y ~ x1 + z1 + x2 + z2 + x1x2 + x1z2 + z1x2 + z1z2 + 
 		strata(strat)), silent=TRUE)
@@ -354,12 +363,12 @@ tdtEpistatic <- function(x1, x2, n.trio, warnError=TRUE){
 	else
 		stat <- pval <- NA
 	out <- list(ll.main=ll.main, ll.full=ll.full, stat=stat, pval=pval,
-		full.model=full) 
+		full.model=full, testtype="epistatic") 
 	class(out) <- "tdtEpi"
 	out
 }
 
-colTDTepistatic <- function(mat.pseudo, n.trio, rn, genes=NULL, valMAF=NULL, warnError=TRUE){
+colTDTepistatic <- function(mat.pseudo, n.trio, rn, genes=NULL, valMAF=NULL){
 	x <- mat.pseudo - 1 
 	z <- (x == 0) - 0.5
 	mat.ids <- cbind(rep.int(rep(1:4, e=4), n.trio), rep.int(1:4, 4*n.trio))
@@ -380,11 +389,8 @@ colTDTepistatic <- function(mat.pseudo, n.trio, rn, genes=NULL, valMAF=NULL, war
 		combs <- allBetweenCombs(ids.genes)
 		ll.main <- ll.full <- numeric(nrow(combs))
 	}
-	if(warnError){
-		wa <- options()$warn
-		on.exit(options(warn=wa))
-		options(warn=2)
-	}
+	wa <- options()$warn
+	options(warn=2)
 	for(i in 1:nrow(combs)){
 		x1 <- x[mat.ids[,1], combs[i,1]]
 		z1 <- z[mat.ids[,1], combs[i,1]] 
@@ -396,12 +402,10 @@ colTDTepistatic <- function(mat.pseudo, n.trio, rn, genes=NULL, valMAF=NULL, war
 		ll.main[i] <- if(is(woIA, "try-error")) NA else woIA$loglik[2]
 		ll.full[i] <- if(is(full, "try-error")) NA else full$loglik[2]
 	}
-	if(warnError)
-		options(warn=wa)
+	options(warn=wa)
 	if(any(is.na(ll.full)) | any(is.na(ll.main)))
-		warning("The fitting of some of the models has failed. A possible reason\n",
-			"is that the two respective SNPs might be in (strong) LD.\n",
-			"The corresponding test statistic and the p-value are thus set to NA.",
+		warning("For some interactions, the fitting of at least one of the models has failed.\n",
+			"Therefore, the corresponding test statistic and the p-value are thus set to NA.",
 			call.=FALSE)
 	stat <- -2 * (ll.main - ll.full)
 	pval <- pchisq(stat, 4, lower.tail=FALSE)
@@ -426,19 +430,67 @@ colTDTepistatic <- function(mat.pseudo, n.trio, rn, genes=NULL, valMAF=NULL, war
 	else
 		mat.maf <- NULL
 	out <- list(ll.main=ll.main, ll.full=ll.full, stat=stat, pval=pval, 
-		maf=valMAF, matMAF=mat.maf, genes=genes, ind.genes=ind.genes)
+		maf=valMAF, matMAF=mat.maf, genes=genes, ind.genes=ind.genes, testtype="epistatic")
 	class(out) <- "colTDTepi"
 	out
 }
-	
+
+
+colGxGlrt <- function(mat.pseudo, mat.ids, combs, y, strat, rn=NULL, genes=NULL, valMAF=NULL){
+	ll.main <- ll.full <- rep.int(NA, nrow(combs))
+	wa <- options()$warn
+	options(warn=2)
+	for(i in 1:nrow(combs)){
+		x1 <- mat.pseudo[mat.ids[,1], combs[i,1]]
+		x2 <- mat.pseudo[mat.ids[,2], combs[i,2]]
+		IA <- x1*x2
+		full <- try(clogit(y ~ x1 + x2 + IA + strata(strat)), silent=TRUE)
+		main <- try(clogit(y ~ x1 + x2 + strata(strat)), silent=TRUE)
+		ll.full[i] <- if(is(full, "try-error")) NA else full$loglik[2]
+		ll.main[i] <- if(is(main, "try-error")) NA else main$loglik[2]
+	}
+	options(warn=wa)
+	if(any(is.na(ll.full)) | any(is.na(ll.main)))
+		warning("For some interactions, the fitting of at least one of the models has failed.\n",
+			"Therefore, the corresponding test statistic and the p-value are thus set to NA.",
+			call.=FALSE)
+	stat <- -2 * (ll.main - ll.full)
+	pval <- pchisq(stat, 4, lower.tail=FALSE)
+	if(any(pval <= .Machine$double.eps, na.rm=TRUE))
+		warning("Some of the p-values are very small (< 2.2e-16). These results might be\n",
+			"misleading, as the small p-values might be caused by non-biological artefacts\n",
+			"(e.g., sparseness of the data).", call.=FALSE)
+	if(is.null(rn))
+		rn <- paste("SNP", 1:ncol(mat.pseudo), sep="")
+	names(ll.full) <- names(stat) <- names(pval) <- paste(rn[combs[,1]], rn[combs[,2]], sep=" : ")
+	if(!is.null(genes)){
+		ind.genes <- genes
+		genes <- paste(genes[combs[,1]], genes[combs[,2]], sep=" : ")
+	}
+	else
+		ind.genes <- NULL
+	if(!is.null(valMAF)){
+		mat.maf <- cbind(round(valMAF, 4)[combs[,1]], round(valMAF, 4)[combs[,2]])
+		rownames(mat.maf) <- names(ll.full)
+		colnames(mat.maf) <- c("First MAF", "Second MAF") 
+	}
+	else
+		mat.maf <- NULL
+	out <- list(ll.main=ll.main, ll.full=ll.full, stat=stat, pval=pval, 
+		maf=valMAF, matMAF=mat.maf, genes=genes, ind.genes=ind.genes, testtype="lrt")
+	class(out) <- "colTDTepi"
+	out	
+}	
 
 print.tdtEpi <- function(x, digits=3, ...){
 	pval <- format.pval(x$pval, digits=digits-1)
-	cat("Genotypic TDT for Epistatic Interactions (Using 15 Pseudo Controls)", 
-		"\n\n", sep="")
-	cat("Likelihood Ratio Test:\n", sep="")
+	if(x$testtype=="epistatic")
+		cat("Likelihood Ratio Test for Epistatic Interactions Based on Genotypic TDTs", 
+			"\n\n", sep="")
+	else
+		cat("Likelihood Ratio Test Based on Genotypic TDTs\n", sep="")
 	if(is.na(x$stat))
-		cat("Failed. A possible reason is that the SNPs might be in (strong) LD.",
+		cat("Failed. At least one of the models did not fit properly",
 			"\n\n")
 	else{
 		cat("Loglikelihood (with Interactions): ", round(x$ll.full, digits), "\n", sep="")
@@ -456,8 +508,8 @@ print.colTDTepi <- function(x, top=5, digits=4, ...){
 		out <- data.frame(out, Genes=x$genes, check.names=FALSE, stringsAsFactors=FALSE)
 	if(!is.null(x$matMAF))
 		out <- data.frame(out, x$matMAF, check.names=FALSE, stringsAsFactors=FALSE) 
-	cat("      Genotypic TDT for Epistatic Interactions (Using 15 Pseudo Controls)",
-		"\n\n")
+	cat("      Genotypic TDT for", ifelse(x$testtype=="epistatic", "Epistatic", "SNP-SNP"), 
+		"Interactions (Using 15 Pseudo Controls)", "\n\n")
 	if(length(x$ll.main) > top){
 		ord <- order(x$pval)[1:top]
 		out <- out[ord,]
@@ -518,6 +570,6 @@ getMatPseudo <- function(mat.snp){
 		}
 		mat.pseudo[,i] <- as.vector(mat.code[,code])
 	}
-	colnames(mat.pseudo)
+	colnames(mat.pseudo) <- colnames(mat.snp)
 	mat.pseudo
 }
